/*
 # ***** BEGIN GPL LICENSE BLOCK *****
 #
 # Copyright (C) 2010: Adrian Boeing. http://www.adrianboeing.com/
 #
 # Thanks to Stan Melax for the original convex hull generation code,
 # and John Ratclif for the C++ version. The C++ code was modified by
 # Adrian Boeing for the PAL project. 
 # http://pal.sf.net/
 #
 # The Blender conversion was writen by Adrian Boeing for Transmin Pty Ltd 
 # http://www.transmin.com.au/
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 # ***** END GPL LICENCE BLOCK *****
 */

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <float.h>
#include "hull.h"
#include <vector>

int main(int argc, char *argv[]) {
    if (argc<2) {
        printf("enter obj file to convert to convex\n");
        return 0;
    }
    
    std::vector<double> srcVerts;
    
    FILE *fin = fopen(argv[1],"r");
    if (!fin) {
        printf("could not open file\n");
        return 0;
    }
    
    float scaleX = 1;
    float scaleY = 1;
    float scaleZ = 1;
    
    float minX = FLT_MAX;
    float minY = FLT_MAX;
    float minZ = FLT_MAX;
    
    float maxX = FLT_MIN;
    float maxY = FLT_MIN;
    float maxZ = FLT_MIN;
    
    while (!feof(fin)) {
        char buffer[256];
		memset(buffer, 0, sizeof(buffer));
		if (!fgets(buffer, 255, fin)) {
//            printf("Potential error reading '%s'\n", argv[1]);
            break;
        }
        //First character specifies load type
        switch (buffer[0]) {
			case '#': //comment, ignore
                break;
            case 'v': //vertex!
                if (buffer[1]!=' ')  //some format we don't want
                    break;
                float x,y,z;
                sscanf(buffer+1,"%f %f %f", &x, &y, &z);
                x*=scaleX;
                y*=scaleY;
                z*=scaleZ;
                
                if (x < minX)
                    minX = x;
                if (y < minY)
                    minY = y;
                if (z < minZ)
                    minZ = z;
                
                if (x > maxX)
                    maxX = x;
                if (y > maxY)
                    maxY = y;
                if (z > maxZ)
                    maxZ = z;
                
                
				srcVerts.push_back(x);
				srcVerts.push_back(y);
				srcVerts.push_back(z);
                break;
            default:
                break;
        }
    }
    
/*    printf("min x,y,z: %f, %f, %f\n",minX,minY,minZ);
    printf("max x,y,z: %f, %f, %f\n",maxX,maxY,maxZ); */
    
    HullDesc desc;
	desc.SetHullFlag(QF_TRIANGLES);
	desc.mVcount       = srcVerts.size()/3;
	desc.mVertices     = &srcVerts[0]; 
	desc.mVertexStride = sizeof(double)*3;
    
	HullResult dresult;
	HullLibrary hl;
	// TODO should something be done if this returns an error?
	/*HullError ret = */
	hl.CreateConvexHull(desc,dresult);
    
    minX = FLT_MAX;
    minY = FLT_MAX;
    minZ = FLT_MAX;
    
    maxX = FLT_MIN;
    maxY = FLT_MIN;
    maxZ = FLT_MIN;
    
    FILE *fout = fopen("out.obj","w");
    for (int i=0;i < dresult.mNumOutputVertices; i++) {
        float x,y,z;
        x = dresult.mOutputVertices[i*3+0];
        y = dresult.mOutputVertices[i*3+1];
        z = dresult.mOutputVertices[i*3+2];
        
        fprintf(fout,"v %f %f %f\n",
            x,y,z);
    }
    
    fprintf(fout,"s off\n");
    for (int i=0;i < dresult.mNumFaces; i++) {
        fprintf(fout,"f %d %d %d\n", 
                dresult.mIndices[i*3+0]+1,
                dresult.mIndices[i*3+1]+1,
                dresult.mIndices[i*3+2]+1);
    }
    
}